/*
 * Copyright 2011 Federal Chancellery Austria and
 * Graz University of Technology
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package at.gv.util.ssl;

import java.util.Properties;

import javax.net.ssl.SSLContext;

import at.gv.util.ex.EgovUtilException;

/**
 * JAX-WS SSL configuration implementation based on Java properties.
 * 
 * @author <a href="mailto:Arne.Tauber@egiz.gv.at">Arne Tauber</a>
 *
 */
public class JaxWsSSLConfigurationPropertiesImpl implements JaxWsSSLConfiguration {

	private Properties props = null;
	private String configDir = null;
	private String propsPrefix = null;
	
	public JaxWsSSLConfigurationPropertiesImpl(Properties props, String configDir, String propsPrefix) {
		if (props == null) {
			throw new NullPointerException("Argument 'props' must not be null.");
		}
		if (configDir == null) {
			throw new NullPointerException("Argument 'configDir' must not be null.");
		}
		if (propsPrefix == null) {
			throw new NullPointerException("Argument 'propsPrefix' must not be null.");
		}
		this.props = props;
		this.configDir = configDir;
		this.propsPrefix = propsPrefix;
	}
	
	public SSLContext getSSLContext(boolean forceTrustAllManager)
	    throws EgovUtilException {
		return SSLUtils.getPropertiesSSLContext(this.props, this.configDir, this.propsPrefix, forceTrustAllManager);
	}

	public boolean useLaxHostNameVerifier() {
		return SSLUtils.useLaxHostNameVerifier(this.props, this.propsPrefix);
	}

}
